@@ -755,24 +755,24 @@ func (s ActivePods) Less(i, j int) bool {
 // length.  After sorting, the pods will be ordered as follows, applying each
 // rule in turn until one matches:
 //
-// 1. If only one of the pods is assigned to a node, the pod that is not
-//    assigned comes before the pod that is.
-// 2. If the pods' phases differ, a pending pod comes before a pod whose phase
-//    is unknown, and a pod whose phase is unknown comes before a running pod.
-// 3. If exactly one of the pods is ready, the pod that is not ready comes
-//    before the ready pod.
-// 4. If controller.kubernetes.io/pod-deletion-cost annotation is set, then
-//    the pod with the lower value will come first.
-// 5. If the pods' ranks differ, the pod with greater rank comes before the pod
-//    with lower rank.
-// 6. If both pods are ready but have not been ready for the same amount of
-//    time, the pod that has been ready for a shorter amount of time comes
-//    before the pod that has been ready for longer.
-// 7. If one pod has a container that has restarted more than any container in
-//    the other pod, the pod with the container with more restarts comes
-//    before the other pod.
-// 8. If the pods' creation times differ, the pod that was created more recently
-//    comes before the older pod.
+//  1. If only one of the pods is assigned to a node, the pod that is not
+//     assigned comes before the pod that is.
+//  2. If the pods' phases differ, a pending pod comes before a pod whose phase
+//     is unknown, and a pod whose phase is unknown comes before a running pod.
+//  3. If exactly one of the pods is ready, the pod that is not ready comes
+//     before the ready pod.
+//  4. If controller.kubernetes.io/pod-deletion-cost annotation is set, then
+//     the pod with the lower value will come first.
+//  5. If the pods' ranks differ, the pod with greater rank comes before the pod
+//     with lower rank.
+//  6. If both pods are ready but have not been ready for the same amount of
+//     time, the pod that has been ready for a shorter amount of time comes
+//     before the pod that has been ready for longer.
+//  7. If one pod has a container that has restarted more than any container in
+//     the other pod, the pod with the container with more restarts comes
+//     before the other pod.
+//  8. If the pods' creation times differ, the pod that was created more recently
+//     comes before the older pod.
 //
 // In 6 and 8, times are compared in a logarithmic scale. This allows a level
 // of randomness among equivalent Pods when sorting. If two pods have the same
@@ -1127,14 +1127,9 @@ func RemoveTaintOffNode(ctx context.Context, c clientset.Interface, nodeName str
 
 // PatchNodeTaints patches node's taints.
 func PatchNodeTaints(ctx context.Context, c clientset.Interface, nodeName string, oldNode *v1.Node, newNode *v1.Node) error {
-	// Strip base diff node from RV to ensure that our Patch request will set RV to check for conflicts over .spec.taints.
-	// This is needed because .spec.taints does not specify patchMergeKey and patchStrategy and adding them is no longer an option for compatibility reasons.
-	// Using other Patch strategy works for adding new taints, however will not resolve problem with taint removal.
-	oldNodeNoRV := oldNode.DeepCopy()
-	oldNodeNoRV.ResourceVersion = ""
-	oldDataNoRV, err := json.Marshal(&oldNodeNoRV)
+	oldData, err := json.Marshal(oldNode)
 	if err != nil {
-		return fmt.Errorf("failed to marshal old node %#v for node %q: %v", oldNodeNoRV, nodeName, err)
+		return fmt.Errorf("failed to marshal old node %#v for node %q: %v", oldNode, nodeName, err)
 	}
 
 	newTaints := newNode.Spec.Taints
@@ -1145,7 +1140,7 @@ func PatchNodeTaints(ctx context.Context, c clientset.Interface, nodeName string
 		return fmt.Errorf("failed to marshal new node %#v for node %q: %v", newNodeClone, nodeName, err)
 	}
 
-	patchBytes, err := strategicpatch.CreateTwoWayMergePatch(oldDataNoRV, newData, v1.Node{})
+	patchBytes, err := strategicpatch.CreateTwoWayMergePatch(oldData, newData, v1.Node{})
 	if err != nil {
 		return fmt.Errorf("failed to create patch for node %q: %v", nodeName, err)
 	}
