@@ -434,10 +434,12 @@ func TestClusterIPMetrics(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error creating CidrSet: %v", err)
 	}
+	a.EnableMetrics()
 	// create IPv6 allocator
 	cidrIPv6 := "2001:db8::/112"
 	_, clusterCIDRv6, _ := netutils.ParseCIDRSloppy(cidrIPv6)
 	b, err := NewInMemory(clusterCIDRv6)
+	b.EnableMetrics()
 	if err != nil {
 		t.Fatalf("unexpected error creating CidrSet: %v", err)
 	}
@@ -546,6 +548,7 @@ func TestClusterIPAllocatedMetrics(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error creating CidrSet: %v", err)
 	}
+	a.EnableMetrics()
 
 	em := testMetrics{
 		free:      0,
@@ -595,6 +598,58 @@ func TestClusterIPAllocatedMetrics(t *testing.T) {
 	}
 }
 
+func TestMetricsDisabled(t *testing.T) {
+	// create metrics enabled allocator
+	cidrIPv4 := "10.0.0.0/24"
+	_, clusterCIDRv4, _ := netutils.ParseCIDRSloppy(cidrIPv4)
+	a, err := NewInMemory(clusterCIDRv4)
+	if err != nil {
+		t.Fatalf("unexpected error creating CidrSet: %v", err)
+	}
+	a.EnableMetrics()
+
+	// create metrics disabled allocator with same CIDR
+	// this metrics should be ignored
+	b, err := NewInMemory(clusterCIDRv4)
+	if err != nil {
+		t.Fatalf("unexpected error creating CidrSet: %v", err)
+	}
+
+	// Check initial state
+	em := testMetrics{
+		free:      0,
+		used:      0,
+		allocated: 0,
+		errors:    0,
+	}
+	expectMetrics(t, cidrIPv4, em)
+
+	// allocate in metrics enabled allocator
+	for i := 0; i < 100; i++ {
+		_, err := a.AllocateNext()
+		if err != nil {
+			t.Fatal(err)
+		}
+	}
+	em = testMetrics{
+		free:      154,
+		used:      100,
+		allocated: 100,
+		errors:    0,
+	}
+	expectMetrics(t, cidrIPv4, em)
+
+	// allocate in metrics disabled allocator
+	for i := 0; i < 200; i++ {
+		_, err := b.AllocateNext()
+		if err != nil {
+			t.Fatal(err)
+		}
+	}
+	// the metrics should not be changed
+	expectMetrics(t, cidrIPv4, em)
+}
+
 // Metrics helpers
 func clearMetrics() {
 	clusterIPAllocated.Reset()
