@@ -20,10 +20,9 @@ package feature
 // This struct allows us to break the dependency of the plugins on
 // the internal k8s features pkg.
 type Features struct {
-	EnablePodAffinityNamespaceSelector bool
-	EnablePodDisruptionBudget          bool
-	EnablePodOverhead                  bool
-	EnableReadWriteOncePod             bool
-	EnableVolumeCapacityPriority       bool
-	EnableCSIStorageCapacity           bool
+	EnableReadWriteOncePod                       bool
+	EnableVolumeCapacityPriority                 bool
+	EnableMinDomainsInPodTopologySpread          bool
+	EnableNodeInclusionPolicyInPodTopologySpread bool
+	EnableMatchLabelKeysInPodTopologySpread      bool
 }
