@@ -22,7 +22,7 @@ import (
 	"fmt"
 	"text/tabwriter"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 
 	authorizationv1 "k8s.io/api/authorization/v1"
