@@ -20,7 +20,7 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 
 	v1 "k8s.io/api/core/v1"
@@ -36,6 +36,7 @@ import (
 	e2evolume "k8s.io/kubernetes/test/e2e/framework/volume"
 	storageframework "k8s.io/kubernetes/test/e2e/storage/framework"
 	storageutils "k8s.io/kubernetes/test/e2e/storage/utils"
+	admissionapi "k8s.io/pod-security-admission/api"
 )
 
 type ephemeralTestSuite struct {
@@ -53,9 +54,9 @@ func InitCustomEphemeralTestSuite(patterns []storageframework.TestPattern) stora
 	}
 }
 
-// InitEphemeralTestSuite returns ephemeralTestSuite that implements TestSuite interface
-// using test suite default patterns
-func InitEphemeralTestSuite() storageframework.TestSuite {
+// GenericEphemeralTestPatterns returns the test patterns for
+// generic ephemeral inline volumes.
+func GenericEphemeralTestPatterns() []storageframework.TestPattern {
 	genericLateBinding := storageframework.DefaultFsGenericEphemeralVolume
 	genericLateBinding.Name += " (late-binding)"
 	genericLateBinding.BindingMode = storagev1.VolumeBindingWaitForFirstConsumer
@@ -64,13 +65,31 @@ func InitEphemeralTestSuite() storageframework.TestSuite {
 	genericImmediateBinding.Name += " (immediate-binding)"
 	genericImmediateBinding.BindingMode = storagev1.VolumeBindingImmediate
 
-	patterns := []storageframework.TestPattern{
-		storageframework.DefaultFsCSIEphemeralVolume,
+	return []storageframework.TestPattern{
 		genericLateBinding,
 		genericImmediateBinding,
+		storageframework.BlockVolModeGenericEphemeralVolume,
 	}
+}
 
-	return InitCustomEphemeralTestSuite(patterns)
+// CSIEphemeralTestPatterns returns the test patterns for
+// CSI ephemeral inline volumes.
+func CSIEphemeralTestPatterns() []storageframework.TestPattern {
+	return []storageframework.TestPattern{
+		storageframework.DefaultFsCSIEphemeralVolume,
+	}
+}
+
+// AllEphemeralTestPatterns returns all pre-defined test patterns for
+// generic and CSI ephemeral inline volumes.
+func AllEphemeralTestPatterns() []storageframework.TestPattern {
+	return append(GenericEphemeralTestPatterns(), CSIEphemeralTestPatterns()...)
+}
+
+// InitEphemeralTestSuite returns ephemeralTestSuite that implements TestSuite interface
+// using test suite default patterns
+func InitEphemeralTestSuite() storageframework.TestSuite {
+	return InitCustomEphemeralTestSuite(AllEphemeralTestPatterns())
 }
 
 func (p *ephemeralTestSuite) GetTestSuiteInfo() storageframework.TestSuiteInfo {
@@ -78,6 +97,9 @@ func (p *ephemeralTestSuite) GetTestSuiteInfo() storageframework.TestSuiteInfo {
 }
 
 func (p *ephemeralTestSuite) SkipUnsupportedTests(driver storageframework.TestDriver, pattern storageframework.TestPattern) {
+	if pattern.VolMode == v1.PersistentVolumeBlock {
+		skipTestIfBlockNotSupported(driver)
+	}
 }
 
 func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pattern storageframework.TestPattern) {
@@ -96,12 +118,16 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 	// Beware that it also registers an AfterEach which renders f unusable. Any code using
 	// f must run inside an It or Context callback.
 	f := framework.NewFrameworkWithCustomTimeouts("ephemeral", storageframework.GetDriverTimeouts(driver))
+	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
 
 	init := func() {
 		if pattern.VolType == storageframework.CSIInlineVolume {
 			eDriver, _ = driver.(storageframework.EphemeralTestDriver)
 		}
 		if pattern.VolType == storageframework.GenericEphemeralVolume {
+			// The GenericEphemeralVolume feature is GA, but
+			// perhaps this test is run against an older Kubernetes
+			// where the feature might be disabled.
 			enabled, err := GenericEphemeralVolumesEnabled(f.ClientSet, f.Timeouts, f.Namespace.Name)
 			framework.ExpectNoError(err, "check GenericEphemeralVolume feature")
 			if !enabled {
@@ -147,12 +173,21 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 	}
 
 	ginkgo.It("should create read-only inline ephemeral volume", func() {
+		if pattern.VolMode == v1.PersistentVolumeBlock {
+			e2eskipper.Skipf("raw block volumes cannot be read-only")
+		}
+
 		init()
 		defer cleanup()
 
 		l.testCase.ReadOnly = true
 		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
-			e2evolume.VerifyExecInPodSucceed(f, pod, "mount | grep /mnt/test | grep ro,")
+			command := "mount | grep /mnt/test | grep ro,"
+			if framework.NodeOSDistroIs("windows") {
+				// attempt to create a dummy file and expect for it not to be created
+				command = "ls /mnt/test* && (touch /mnt/test-0/hello-world || true) && [ ! -f /mnt/test-0/hello-world ]"
+			}
+			e2evolume.VerifyExecInPodSucceed(f, pod, command)
 			return nil
 		}
 		l.testCase.TestEphemeral()
@@ -164,13 +199,82 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 
 		l.testCase.ReadOnly = false
 		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
-			e2evolume.VerifyExecInPodSucceed(f, pod, "mount | grep /mnt/test | grep rw,")
+			command := "mount | grep /mnt/test | grep rw,"
+			if framework.NodeOSDistroIs("windows") {
+				// attempt to create a dummy file and expect for it to be created
+				command = "ls /mnt/test* && touch /mnt/test-0/hello-world && [ -f /mnt/test-0/hello-world ]"
+			}
+			if pattern.VolMode == v1.PersistentVolumeBlock {
+				command = "if ! [ -b /mnt/test-0 ]; then echo /mnt/test-0 is not a block device; exit 1; fi"
+			}
+			e2evolume.VerifyExecInPodSucceed(f, pod, command)
 			return nil
 		}
 		l.testCase.TestEphemeral()
 	})
 
-	ginkgo.It("should support two pods which share the same volume", func() {
+	ginkgo.It("should support expansion of pvcs created for ephemeral pvcs", func() {
+		if pattern.VolType != storageframework.GenericEphemeralVolume {
+			e2eskipper.Skipf("Skipping %s test for expansion", pattern.VolType)
+		}
+
+		init()
+		defer cleanup()
+
+		if !driver.GetDriverInfo().Capabilities[storageframework.CapOnlineExpansion] {
+			e2eskipper.Skipf("Driver %q does not support online volume expansion - skipping", driver.GetDriverInfo().Name)
+		}
+
+		l.testCase.ReadOnly = false
+		l.testCase.RunningPodCheck = func(pod *v1.Pod) interface{} {
+			podName := pod.Name
+			framework.Logf("Running volume expansion checks %s", podName)
+
+			outerPodVolumeSpecName := ""
+			for i := range pod.Spec.Volumes {
+				volume := pod.Spec.Volumes[i]
+				if volume.Ephemeral != nil {
+					outerPodVolumeSpecName = volume.Name
+					break
+				}
+			}
+			pvcName := fmt.Sprintf("%s-%s", podName, outerPodVolumeSpecName)
+			pvc, err := f.ClientSet.CoreV1().PersistentVolumeClaims(pod.Namespace).Get(context.TODO(), pvcName, metav1.GetOptions{})
+			framework.ExpectNoError(err, "error getting ephemeral pvc")
+
+			ginkgo.By("Expanding current pvc")
+			currentPvcSize := pvc.Spec.Resources.Requests[v1.ResourceStorage]
+			newSize := currentPvcSize.DeepCopy()
+			newSize.Add(resource.MustParse("1Gi"))
+			framework.Logf("currentPvcSize %s, requested new size %s", currentPvcSize.String(), newSize.String())
+
+			newPVC, err := ExpandPVCSize(pvc, newSize, f.ClientSet)
+			framework.ExpectNoError(err, "While updating pvc for more size")
+			pvc = newPVC
+			gomega.Expect(pvc).NotTo(gomega.BeNil())
+
+			pvcSize := pvc.Spec.Resources.Requests[v1.ResourceStorage]
+			if pvcSize.Cmp(newSize) != 0 {
+				framework.Failf("error updating pvc %s from %s to %s size", pvc.Name, currentPvcSize.String(), newSize.String())
+			}
+
+			ginkgo.By("Waiting for cloudprovider resize to finish")
+			err = WaitForControllerVolumeResize(pvc, f.ClientSet, totalResizeWaitPeriod)
+			framework.ExpectNoError(err, "While waiting for pvc resize to finish")
+
+			ginkgo.By("Waiting for file system resize to finish")
+			pvc, err = WaitForFSResize(pvc, f.ClientSet)
+			framework.ExpectNoError(err, "while waiting for fs resize to finish")
+
+			pvcConditions := pvc.Status.Conditions
+			framework.ExpectEqual(len(pvcConditions), 0, "pvc should not have conditions")
+			return nil
+		}
+		l.testCase.TestEphemeral()
+
+	})
+
+	ginkgo.It("should support two pods which have the same volume definition", func() {
 		init()
 		defer cleanup()
 
@@ -195,8 +299,8 @@ func (p *ephemeralTestSuite) DefineTests(driver storageframework.TestDriver, pat
 			// between pods, then we can check whether
 			// data written in one pod is really not
 			// visible in the other.
-			if !readOnly && !shared {
-				ginkgo.By("writing data in one pod and checking for it in the second")
+			if pattern.VolMode != v1.PersistentVolumeBlock && !readOnly && !shared {
+				ginkgo.By("writing data in one pod and checking the second does not see it (it should get its own volume)")
 				e2evolume.VerifyExecInPodSucceed(f, pod, "touch /mnt/test-0/hello-world")
 				e2evolume.VerifyExecInPodSucceed(f, pod2, "[ ! -f /mnt/test-0/hello-world ]")
 			}
@@ -272,7 +376,8 @@ func (t EphemeralTest) TestEphemeral() {
 	gomega.Expect(client).NotTo(gomega.BeNil(), "EphemeralTest.Client is required")
 
 	ginkgo.By(fmt.Sprintf("checking the requested inline volume exists in the pod running on node %+v", t.Node))
-	command := "mount | grep /mnt/test && sleep 10000"
+	command := "sleep 10000"
+
 	var volumes []v1.VolumeSource
 	numVolumes := t.NumInlineVolumes
 	if numVolumes == 0 {
@@ -346,9 +451,10 @@ func StartInPodWithInlineVolume(c clientset.Interface, ns, podName, command stri
 		Spec: v1.PodSpec{
 			Containers: []v1.Container{
 				{
-					Name:    "csi-volume-tester",
-					Image:   e2epod.GetDefaultTestImage(),
-					Command: e2epod.GenerateScriptCmd(command),
+					Name:  "csi-volume-tester",
+					Image: e2epod.GetDefaultTestImage(),
+					// NOTE: /bin/sh works on both agnhost and busybox
+					Command: []string{"/bin/sh", "-c", command},
 				},
 			},
 			RestartPolicy: v1.RestartPolicyNever,
@@ -358,12 +464,22 @@ func StartInPodWithInlineVolume(c clientset.Interface, ns, podName, command stri
 
 	for i, volume := range volumes {
 		name := fmt.Sprintf("my-volume-%d", i)
-		pod.Spec.Containers[0].VolumeMounts = append(pod.Spec.Containers[0].VolumeMounts,
-			v1.VolumeMount{
-				Name:      name,
-				MountPath: fmt.Sprintf("/mnt/test-%d", i),
-				ReadOnly:  readOnly,
-			})
+		path := fmt.Sprintf("/mnt/test-%d", i)
+		if volume.Ephemeral != nil && volume.Ephemeral.VolumeClaimTemplate.Spec.VolumeMode != nil &&
+			*volume.Ephemeral.VolumeClaimTemplate.Spec.VolumeMode == v1.PersistentVolumeBlock {
+			pod.Spec.Containers[0].VolumeDevices = append(pod.Spec.Containers[0].VolumeDevices,
+				v1.VolumeDevice{
+					Name:       name,
+					DevicePath: path,
+				})
+		} else {
+			pod.Spec.Containers[0].VolumeMounts = append(pod.Spec.Containers[0].VolumeMounts,
+				v1.VolumeMount{
+					Name:      name,
+					MountPath: path,
+					ReadOnly:  readOnly,
+				})
+		}
 		pod.Spec.Volumes = append(pod.Spec.Volumes,
 			v1.Volume{
 				Name:         name,
